#!/usr/bin/env python

from hapi import fetch,db_begin,getStickXY,select,volumeConcentration,\
                 fetch_by_ids,absorptionCoefficient_Lorentz,read_hotw,\
                 convolveSpectrum,SLIT_RECTANGULAR,tableList
from pylab import plot,subplot,show,hold,xlabel,ylabel,\
                  title,xlim,ylim,rcParams,legend,grid,xticks,yticks
rcParams['figure.figsize'] = 17,6
from numpy import log10,log,exp
from os.path import exists

CASES = ['NO2','H2O2','C2H4','CH3OH','CH3BR','HNO3','H2CO','HCOOH','HO2']
TEMPERATURES={\
    'NO2':25.,'H2O2':25.,'C2H4':5.,'CH3OH':5.,'CH3BR':5.,
    'HNO3':5.,'H2CO':5.,'HCOOH':25.,'HO2':5.,
    }

# TEMPERATURE
t_zero = -273.15; # absolute zero in Celsius
PRES = 1 # pressure in atmospheres

# INITIALIZE ------------------------
def fetch_molecule_NO2():
    db_begin('NO2')
    fetch_by_ids('NO2',[44,],500,3500)

def fetch_molecule_H2O2():
    db_begin('H2O2')
    fetch_by_ids('H2O2',[75,],50,2000)

def fetch_molecule_C2H4():
    db_begin('C2H4')
    fetch_by_ids('C2H4',[90,91],300,3000)

def fetch_molecule_CH3OH():
    db_begin('CH3OH')
    fetch_by_ids('CH3OH',[92,],300,3500)

def fetch_molecule_CH3BR():
    db_begin('CH3BR')
    fetch_by_ids('CH3BR',[93,94],300,6500)

def fetch_molecule_HNO3():
    db_begin('HNO3')
    fetch_by_ids('HNO3',[47,],500,2000)

def fetch_molecule_H2CO():
    db_begin('H2CO')
    fetch_by_ids('H2CO',[64,65,66],300,3500)

def fetch_molecule_HCOOH():
    db_begin('HCOOH')
    fetch_by_ids('HCOOH',[84,],300,3000)

def fetch_molecule_HO2():
    db_begin('HO2')
    fetch_by_ids('HO2',[85,],0,2000)

def fetch_molecule(CaseName):
    CaseName = CaseName.upper()
    if CaseName == 'NO2':
        fetch_molecule_NO2()
    elif CaseName == 'H2O2':
        fetch_molecule_H2O2()
    elif CaseName == 'C2H4':
        fetch_molecule_C2H4()
    elif CaseName == 'CH3OH':
        fetch_molecule_CH3OH()
    elif CaseName == 'CH3BR':
        fetch_molecule_CH3BR()
    elif CaseName == 'HNO3':
        fetch_molecule_HNO3()
    elif CaseName == 'H2CO':
        fetch_molecule_H2CO()
    elif CaseName == 'HCOOH':
        fetch_molecule_HCOOH()
    elif CaseName == 'HO2':
        fetch_molecule_HO2()

def init(CaseName):
    prefix = CaseName+'/'+CaseName
    filename_data = prefix+'.data'
    filename_header = prefix+'.header'
    if exists(filename_data) and exists(filename_header):
        db_begin(CaseName)
    else:
        fetch_molecule(CaseName)
# /INITIALIZE -----------------------
    
# CALC SPECTRUM ---------------------
def calc_spectrum(CaseName,TEMP):
    nu_HO,coef_HO = absorptionCoefficient_Lorentz(SourceTables=CaseName,
                                        Environment={'T':TEMP,'p':1.0})
    nu_conv,coef_conv,d1,d2,d3 = convolveSpectrum(nu_HO,coef_HO,Resolution=0.112,
                                    SlitFunction=SLIT_RECTANGULAR,AF_wing=5)
    return nu_conv,coef_conv
                    
def calc_NO2(TEMP):
    return calc_spectrum('NO2',TEMP)

def calc_H2O2(TEMP):
    return calc_spectrum('H2O2',TEMP)

def calc_C2H4(TEMP):
    return calc_spectrum('C2H4',TEMP)

def calc_CH3OH(TEMP):
    if TEMP+t_zero != 5.: raise Exception('only 5C is allowed for CH3OH')
    partitionSum_CH3OH_5C = lambda M,I,T: 73800. if T<296. else 82469.
    nu_HO,coef_HO = absorptionCoefficient_Lorentz(SourceTables='CH3OH',
                        Environment={'T':TEMP,'p':1.0},
                        partitionFunction=partitionSum_CH3OH_5C)
    nu_conv,coef_conv,d1,d2,d3 = convolveSpectrum(nu_HO,coef_HO,Resolution=0.112,
                                    SlitFunction=SLIT_RECTANGULAR,AF_wing=5)
    return nu_conv,coef_conv

def calc_CH3BR(TEMP):
    return calc_spectrum('CH3BR',TEMP)

def calc_HNO3(TEMP):
    return calc_spectrum('HNO3',TEMP)

def calc_H2CO(TEMP):
    return calc_spectrum('H2CO',TEMP)

def calc_HCOOH(TEMP):
    return calc_spectrum('HCOOH',TEMP)

def calc_HO2(TEMP):
    return calc_spectrum('HO2',TEMP)

def calc(CaseName,TEMP):
    CaseName = CaseName.upper()
    if CaseName == 'NO2':
        return calc_NO2(TEMP)
    elif CaseName == 'H2O2':
        return calc_H2O2(TEMP)
    elif CaseName == 'C2H4':
        return calc_C2H4(TEMP)
    elif CaseName == 'CH3OH':
        return calc_CH3OH(TEMP)
    elif CaseName == 'CH3BR':
        return calc_CH3BR(TEMP)
    elif CaseName == 'HNO3':
        return calc_HNO3(TEMP)
    elif CaseName == 'H2CO':
        return calc_H2CO(TEMP)
    elif CaseName == 'HCOOH':
        return calc_HCOOH(TEMP)
    elif CaseName == 'HO2':
        return calc_HO2(TEMP)
# /CALC SPECTRUM --------------------

# READ OBSERVED DATA ----------------
def read_obs_NO2():
    return read_hotw('NO2_25T.TXT')
    
def read_obs_H2O2():
    return read_hotw('H2O2_25T.TXT')

def read_obs_C2H4():
    return read_hotw('Ethene_5T.TXT')

def read_obs_CH3OH():
    return read_hotw('MeOH_5T.TXT')

def read_obs_CH3BR():
    return read_hotw('CH3BR_5T.TXT')

def read_obs_HNO3():
    return read_hotw('HNO3_5T.TXT')

def read_obs_H2CO():
    return read_hotw('HCHO_5T.TXT')

def read_obs_HCOOH():
    return read_hotw('FACID_25T.TXT')

def read_obs_HO2():
    return [0,0]
    
def read_obs(CaseName):
    CaseName = CaseName.upper()
    if CaseName == 'NO2':
        return read_obs_NO2()
    elif CaseName == 'H2O2':
        return read_obs_H2O2()
    elif CaseName == 'C2H4':
        return read_obs_C2H4()
    elif CaseName == 'CH3OH':
        return read_obs_CH3OH()
    elif CaseName == 'CH3BR':
        return read_obs_CH3BR()
    elif CaseName == 'HNO3':
        return read_obs_HNO3()
    elif CaseName == 'H2CO':
        return read_obs_H2CO()
    elif CaseName == 'HCOOH':
        return read_obs_HCOOH()
    elif CaseName == 'HO2':
        return read_obs_HO2()
# /READ OBSERVED DATA --------------

# CALCULATE AND SAVE RESULTING DATA--
def result_NO2(inp):
    nu_conv,coef_conv,nu_pnnl,coef_pnnl,VOLCON = inp
    rcParams['figure.figsize'] = 17,4
    plot(nu_pnnl,log10(coef_pnnl*2.3*10000/VOLCON),'-g',
         nu_conv,log10(coef_conv),'-r'); 
    legend(['PNNL','HITRAN-2012'],fontsize=15)
    xlim(500,2000)
    ylim([-22,-16])
    xlabel('Wavenumber / $cm^{-1}$',fontsize=15)
    ylabel('Absorption coefficient / $cm^2/molecule$',fontsize=12)
    title('HITRAN-2012 VS PNNL: $^{14}N^{16}O_2$ at T = 25$^o$ C and p = 1 atm',fontsize=15)
    grid(True)
    xticks(fontsize=15)
    yticks(fontsize=15)
    show()

def result_H2O2(inp):
    nu_conv,coef_conv,nu_pnnl,coef_pnnl,VOLCON = inp
    rcParams['figure.figsize'] = 17,4
    plot(nu_pnnl,log10(coef_pnnl*2.3*10000/VOLCON),'-g',
         nu_conv,log10(coef_conv),'-r'); 
    legend(['PNNL','HITRAN-2012'],fontsize=15)
    xlim(500,2000)
    ylim([-22,-16])
    xlabel('Wavenumber / $cm^{-1}$',fontsize=15)
    ylabel('Absorption coefficient / $cm^2/molecule$',fontsize=12)
    title('HITRAN-2012 VS PNNL: $H_2^{16}O_2$ at T = 25$^o$ C and p = 1 atm',fontsize=15)
    grid(True)
    xticks(fontsize=15)
    yticks(fontsize=15)
    show()

def result_C2H4(inp):
    nu_conv,coef_conv,nu_pnnl,coef_pnnl,VOLCON = inp
    rcParams['figure.figsize'] = 17,4
    plot(nu_pnnl,log10(coef_pnnl*2.3*10000/VOLCON),'-g',
     nu_conv,log10(coef_conv),'-r'); 
    legend(['PNNL','HITRAN-2012'],fontsize=15)
    xlim(500,2000)
    ylim([-22,-14])
    xlabel('Wavenumber / $cm^{-1}$',fontsize=15)
    ylabel('Absorption coefficient / $cm^2/molecule$',fontsize=12)
    title('HITRAN-2012 VS PNNL: $^{12}C_2H_4+^{12}CH_2^{13}CH_2$ at T = 5$^o$C and p = 1 atm',fontsize=15)
    grid(True)
    xticks(fontsize=15)
    yticks(fontsize=15)
    show()

def result_CH3OH(inp):
    nu_conv,coef_conv,nu_pnnl,coef_pnnl,VOLCON = inp
    rcParams['figure.figsize'] = 17,4
    plot(nu_pnnl,log10(coef_pnnl*2.3*10000/VOLCON),'-g',
         nu_conv,log10(coef_conv),'-r'); 
    legend(['PNNL','HITRAN-2012'],fontsize=15)
    xlim(500,2000)
    ylim([-22,-16])
    xlabel('Wavenumber / $cm^{-1}$',fontsize=15)
    ylabel('Absorption coefficient / $cm^2/molecule$',fontsize=12)
    title('HITRAN-2012 VS PNNL: $^{12}CH_3^{16}OH$ at T = 5$^o$C and p = 1 atm',fontsize=15)
    grid(True)
    xticks(fontsize=15)
    yticks(fontsize=15)
    show()

def result_CH3BR(inp):
    nu_conv,coef_conv,nu_pnnl,coef_pnnl,VOLCON = inp
    rcParams['figure.figsize'] = 17,4
    plot(nu_pnnl,log10(coef_pnnl*2.3*10000/VOLCON),'-g',
         nu_conv,log10(coef_conv),'-r'); 
    legend(['PNNL','HITRAN-2012'],fontsize=15)
    xlim(500,2000)
    ylim([-22,-16])
    xlabel('Wavenumber / $cm^{-1}$',fontsize=15)
    ylabel('Absorption coefficient / $cm^2/molecule$',fontsize=12)
    title('HITRAN-2012 VS PNNL: $^{12}CH_3^{79}Br+^{12}CH_3^{81}Br$ at T = 5$^o$C and p = 1 atm',fontsize=15)
    grid(True)
    xticks(fontsize=15)
    yticks(fontsize=15)
    show()

def result_HNO3(inp):
    nu_conv,coef_conv,nu_pnnl,coef_pnnl,VOLCON = inp
    rcParams['figure.figsize'] = 17,4
    plot(nu_pnnl,log10(coef_pnnl*2.3*10000/VOLCON),'-g',
         nu_conv,log10(coef_conv),'-r'); 
    legend(['PNNL','HITRAN-2012'],fontsize=15)
    xlim(500,2000)
    ylim([-22,-16])
    xlabel('Wavenumber / $cm^{-1}$',fontsize=15)
    ylabel('Absorption coefficient / $cm^2/molecule$',fontsize=12)
    title('HITRAN-2012 VS PNNL: $H^{14}N^{16}O_3$ at T = 5$^o$C and p = 1 atm',fontsize=15)
    grid(True)
    xticks(fontsize=15)
    yticks(fontsize=15)
    show()

def result_H2CO(inp):
    nu_conv,coef_conv,nu_pnnl,coef_pnnl,VOLCON = inp
    rcParams['figure.figsize'] = 17,4
    plot(nu_pnnl,log10(coef_pnnl*2.3*10000/VOLCON),'-g',
         nu_conv,log10(coef_conv),'-r'); 
    legend(['PNNL','HITRAN-2012'],fontsize=15)
    xlim(500,2000)
    ylim([-22,-16])
    xlabel('Wavenumber / $cm^{-1}$',fontsize=15)
    ylabel('Absorption coefficient / $cm^2/molecule$',fontsize=12)
    title('HITRAN-2012 VS PNNL: $H_2^{12}C^{16}O+H_2^{13}C^{16}O+H_2^{12}C^{18}O$ at T = 5$^o$C and p = 1 atm',fontsize=15)
    grid(True)
    xticks(fontsize=15)
    yticks(fontsize=15)
    show()

def result_HCOOH(inp):
    nu_conv,coef_conv,nu_pnnl,coef_pnnl,VOLCON = inp
    rcParams['figure.figsize'] = 17,4
    plot(nu_pnnl,log10(coef_pnnl*2.3*10000/VOLCON),'-g',
         nu_conv,log10(coef_conv),'-r'); 
    legend(['PNNL','HITRAN-2012'],fontsize=15)
    xlim(500,2000)
    ylim([-22,-16])
    xlabel('Wavenumber / $cm^{-1}$',fontsize=15)
    ylabel('Absorption coefficient / $cm^2/molecule$',fontsize=12)
    title('HITRAN-2012 VS PNNL: $H^{12}C^{16}O^{16}OH$ at T = 25$^o$ C and p = 1 atm',fontsize=15)
    grid(True)
    xticks(fontsize=15)
    yticks(fontsize=15)
    show()

def result_HO2(inp):
    nu_conv,coef_conv,nu_pnnl,coef_pnnl,VOLCON = inp
    rcParams['figure.figsize'] = 17,4
    plot(nu_conv,log10(coef_conv),'-r'); 
    legend(['HITRAN-2012',])
    xlim(500,2000)
    ylim([-22,-17])
    xlabel('Wavenumber / $cm^{-1}$',fontsize=15)
    ylabel('Absorption coefficient / $cm^2/molecule$',fontsize=12)
    title('HITRAN-2012: $H^{16}O_2$ at T = 5$^o$C and p = 1 atm',fontsize=15)
    grid(True)
    xticks(fontsize=15)
    yticks(fontsize=15)
    show()

def result(CaseName,inp):
    CaseName = CaseName.upper()
    if CaseName == 'NO2':
        result_NO2(inp)
    elif CaseName == 'H2O2':
        result_H2O2(inp)
    elif CaseName == 'C2H4':
        result_C2H4(inp)
    elif CaseName == 'CH3OH':
        result_CH3OH(inp)
    elif CaseName == 'CH3BR':
        result_CH3BR(inp)
    elif CaseName == 'HNO3':
        result_HNO3(inp)
    elif CaseName == 'H2CO':
        result_H2CO(inp)
    elif CaseName == 'HCOOH':
        result_HCOOH(inp)
    elif CaseName == 'HO2':
        result_HO2(inp)
# /CALCULATE AND SAVE RESULTING DATA-

SAVE_CROSS_SECTIONS = False

def main():
    """
    """
    while True:
        print('\nAvailable molecules:')
        print(CASES)
        CaseName = raw_input('\nPlease enter molecule name (q to quit): ')
        CaseName = CaseName.upper()
        if CaseName in set(['EXIT','QUIT','Q']):
            break
        if CaseName not in set(CASES):
            print('Specified molecule is not in the list')
            continue
        init(CaseName)
        TEMP = TEMPERATURES[CaseName]-t_zero # in Kelvin
        nu_calc,coef_calc = calc(CaseName,TEMP)
        nu_obs,coef_obs = read_obs(CaseName)
        VOLCON = volumeConcentration(PRES,TEMP)
        result(CaseName,[nu_calc,coef_calc,nu_obs,coef_obs,VOLCON])
        if SAVE_CROSS_SECTIONS:
            f = open(CaseName+'/'+CaseName+'.xsect','w')
            for nu,coef in zip(nu_calc,coef_calc):
                f.write('%e %e\n' % (nu,coef))
            f.close()
        
if __name__ == '__main__':
    main()
